@@ -15,7 +15,7 @@ use druid::{Data, Lens};
 use crate::error::{Error, ErrorKind, GlifError, GlifErrorInternal};
 use crate::names::NameList;
 use crate::shared_types::PUBLIC_OBJECT_LIBS_KEY;
-use crate::{Color, Guideline, Identifier, Line, Plist};
+use crate::{Color, Guideline, Identifier, Line, Plist, WriteOptions};
 
 /// The name of a glyph.
 pub type GlyphName = Arc<str>;
@@ -64,13 +64,20 @@ impl Glyph {
 
     #[doc(hidden)]
     pub fn save<P: AsRef<Path>>(&self, path: P) -> Result<(), Error> {
+        let path = path.as_ref();
+        let opts = WriteOptions::default();
+        self.save_with_options(path, &opts)
+    }
+
+    pub(crate) fn save_with_options(&self, path: &Path, opts: &WriteOptions) -> Result<(), Error> {
         if self.format != GlifVersion::V2 {
             return Err(Error::DowngradeUnsupported);
         }
         if self.lib.contains_key(PUBLIC_OBJECT_LIBS_KEY) {
             return Err(Error::PreexistingPublicObjectLibsKey);
         }
-        let data = self.encode_xml()?;
+
+        let data = self.encode_xml_with_options(opts)?;
         std::fs::write(path, &data)?;
         Ok(())
     }
@@ -203,6 +210,9 @@ impl Data for Glyph {
     }
 }
 
+/// Version of a `.glif` file, per the [UFO spec].
+///
+/// [UFO spec]: https://unifiedfontobject.org/versions/ufo1/glyphs/glif/#specification
 #[derive(Debug, Clone, PartialEq)]
 #[cfg_attr(feature = "druid", derive(Data))]
 pub enum GlifVersion {
@@ -535,6 +545,14 @@ impl ContourPoint {
     pub fn to_kurbo(&self) -> kurbo::Point {
         kurbo::Point::new(self.x as f64, self.y as f64)
     }
+
+    /// Applies a transformation matrix to the point's coordinates
+    pub fn transform(&mut self, transform: AffineTransform) {
+        let new_x = transform.x_scale * self.x + transform.yx_scale * self.y + transform.x_offset;
+        let new_y = transform.xy_scale * self.x + transform.y_scale * self.y + transform.y_offset;
+        self.x = new_x;
+        self.y = new_y;
+    }
 }
 
 impl Component {
