@@ -11,7 +11,7 @@ use rayon::prelude::*;
 use crate::glyph::GlyphName;
 use crate::names::NameList;
 use crate::shared_types::Color;
-use crate::{Error, Glyph, Plist};
+use crate::{util, Error, Glyph, Plist, WriteOptions};
 
 static CONTENTS_FILE: &str = "contents.plist";
 static LAYER_INFO_FILE: &str = "layerinfo.plist";
@@ -32,7 +32,7 @@ pub struct LayerSet {
     layers: Vec<Layer>,
 }
 
-#[allow(clippy::clippy::len_without_is_empty)] // never empty
+#[allow(clippy::len_without_is_empty)] // never empty
 impl LayerSet {
     /// Load the layers from the provided path.
     ///
@@ -53,7 +53,7 @@ impl LayerSet {
             .into_iter()
             .map(|(name, path)| {
                 let layer_path = base_dir.join(&path);
-                Layer::load_impl(&layer_path, name, &glyph_names)
+                Layer::load_impl(&layer_path, name, glyph_names)
             })
             .collect::<Result<_, _>>()?;
 
@@ -79,6 +79,7 @@ impl LayerSet {
     /// The number of layers in the set.
     ///
     /// This should be non-zero.
+    #[allow(clippy::len_without_is_empty)]
     pub fn len(&self) -> usize {
         self.layers.len()
     }
@@ -178,8 +179,9 @@ impl Default for LayerSet {
     }
 }
 
-/// A [layer], corresponding to a 'glyphs' directory. Conceptually, a layer
-/// is just a collection of glyphs.
+/// A [layer], corresponding to a 'glyphs' directory.
+///
+/// Conceptually, a layer is just a collection of glyphs.
 ///
 /// [layer]: http://unifiedfontobject.org/versions/ufo3/glyphs/
 #[derive(Debug, Clone, PartialEq)]
@@ -238,6 +240,9 @@ impl Layer {
         names: &NameList,
     ) -> Result<Layer, Error> {
         let contents_path = path.join(CONTENTS_FILE);
+        if !contents_path.exists() {
+            return Err(Error::MissingFile(contents_path.display().to_string()));
+        }
         // these keys are never used; a future optimization would be to skip the
         // names and deserialize to a vec; that would not be a one-liner, though.
         let contents: BTreeMap<GlyphName, PathBuf> = plist::from_file(contents_path)?;
@@ -300,42 +305,52 @@ impl Layer {
         Ok((color, lib))
     }
 
-    fn layerinfo_to_file(&self, path: &Path) -> Result<(), Error> {
+    fn layerinfo_to_file_if_needed(
+        &self,
+        path: &Path,
+        options: &WriteOptions,
+    ) -> Result<(), Error> {
+        if self.color.is_none() && self.lib.is_empty() {
+            return Ok(());
+        }
+
         let mut dict = plist::dictionary::Dictionary::new();
 
         if let Some(c) = &self.color {
-            dict.insert("color".into(), plist::Value::String(c.to_rgba_string()));
+            dict.insert("color".into(), c.to_rgba_string().into());
         }
         if !self.lib.is_empty() {
-            dict.insert("lib".into(), plist::Value::Dictionary(self.lib.clone()));
+            dict.insert("lib".into(), self.lib.clone().into());
         }
 
-        plist::Value::Dictionary(dict).to_file_xml(path.join(LAYER_INFO_FILE))?;
+        util::recursive_sort_plist_keys(&mut dict);
 
-        Ok(())
-    }
-
-    fn layerinfo_is_empty(&self) -> bool {
-        self.color.is_none() && self.lib.is_empty()
+        crate::write::write_plist_value_to_file(&path.join(LAYER_INFO_FILE), &dict.into(), options)
     }
 
     /// Attempt to write this layer to the given path.
     ///
     /// The path should not exist.
     pub fn save(&self, path: impl AsRef<Path>) -> Result<(), Error> {
-        let path = path.as_ref();
-        fs::create_dir(&path)?;
-        plist::to_file_xml(path.join(CONTENTS_FILE), &self.contents)?;
-        // Avoid writing empty layerinfo.plist file.
-        if !self.layerinfo_is_empty() {
-            self.layerinfo_to_file(&path)?;
-        }
-        for (name, glyph_path) in self.contents.iter() {
-            let glyph = self.glyphs.get(name).expect("all glyphs in contents must exist.");
-            glyph.save(path.join(glyph_path))?;
-        }
+        let options = WriteOptions::default();
+        self.save_with_options(path.as_ref(), &options)
+    }
 
-        Ok(())
+    pub fn save_with_options(&self, path: &Path, opts: &WriteOptions) -> Result<(), Error> {
+        fs::create_dir(&path)?;
+        crate::write::write_xml_to_file(&path.join(CONTENTS_FILE), &self.contents, opts)?;
+
+        self.layerinfo_to_file_if_needed(path, opts)?;
+
+        #[cfg(feature = "rayon")]
+        let iter = self.contents.par_iter();
+        #[cfg(not(feature = "rayon"))]
+        let mut iter = self.contents.iter();
+
+        iter.try_for_each(|(name, glyph_path)| {
+            let glyph = self.glyphs.get(name).expect("all glyphs in contents must exist.");
+            glyph.save_with_options(&path.join(glyph_path), opts)
+        })
     }
 
     /// The number of [`Glyph`]s in the layer.
@@ -432,18 +447,6 @@ impl Layer {
         }
     }
 
-    #[doc(hidden)]
-    #[deprecated(since = "0.4.0", note = "renamed to 'iter'")]
-    pub fn iter_contents(&self) -> impl Iterator<Item = &Arc<Glyph>> + '_ {
-        self.iter()
-    }
-
-    #[doc(hidden)]
-    #[deprecated(since = "0.4.0", note = "renamed to 'iter_mut'")]
-    pub fn iter_contents_mut(&mut self) -> impl Iterator<Item = &mut Glyph> {
-        self.iter_mut()
-    }
-
     /// Iterate over the glyphs in this layer.
     pub fn iter(&self) -> impl Iterator<Item = &Arc<Glyph>> + '_ {
         self.glyphs.values()
@@ -454,7 +457,9 @@ impl Layer {
         self.glyphs.values_mut().map(Arc::make_mut)
     }
 
-    #[cfg(test)]
+    /// Returns the path to the .glif file of a given glyph name.
+    ///
+    /// The returned path is relative to the path of the current layer.
     pub fn get_path(&self, name: &str) -> Option<&Path> {
         self.contents.get(name).map(PathBuf::as_path)
     }
@@ -472,8 +477,9 @@ mod tests {
     use std::path::Path;
 
     #[test]
+    #[allow(clippy::float_cmp)]
     fn load_layer() {
-        let layer_path = "testdata/mutatorSans/MutatorSansBoldWide.ufo/glyphs";
+        let layer_path = "testdata/MutatorSansLightWide.ufo/glyphs";
         assert!(Path::new(layer_path).exists(), "missing test data. Did you `git submodule init`?");
         let layer = Layer::load(layer_path, DEFAULT_LAYER_NAME.into()).unwrap();
         assert_eq!(
@@ -486,13 +492,13 @@ mod tests {
         );
         let glyph = layer.get_glyph("A").expect("failed to load glyph 'A'");
         assert_eq!(glyph.height, 0.);
-        assert_eq!(glyph.width, 1290.);
+        assert_eq!(glyph.width, 1190.);
         assert_eq!(glyph.codepoints, vec!['A']);
     }
 
     #[test]
     fn load_write_layerinfo() {
-        let layer_path = "testdata/mutatorSans/MutatorSansBoldWide.ufo/glyphs";
+        let layer_path = "testdata/MutatorSansLightWide.ufo/glyphs";
         assert!(Path::new(layer_path).exists(), "missing test data. Did you `git submodule init`?");
         let mut layer = Layer::load(layer_path, DEFAULT_LAYER_NAME.into()).unwrap();
 
@@ -534,7 +540,7 @@ mod tests {
 
     #[test]
     fn delete() {
-        let layer_path = "testdata/mutatorSans/MutatorSansBoldWide.ufo/glyphs";
+        let layer_path = "testdata/MutatorSansLightWide.ufo/glyphs";
         let mut layer = Layer::load(layer_path, DEFAULT_LAYER_NAME.into()).unwrap();
         layer.remove_glyph("A");
         if let Some(glyph) = layer.get_glyph("A") {
@@ -547,8 +553,9 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::float_cmp)]
     fn set_glyph() {
-        let layer_path = "testdata/mutatorSans/MutatorSansBoldWide.ufo/glyphs";
+        let layer_path = "testdata/MutatorSansLightWide.ufo/glyphs";
         let mut layer = Layer::load(layer_path, DEFAULT_LAYER_NAME.into()).unwrap();
         let mut glyph = Glyph::new_named("A");
         glyph.width = 69.;
@@ -559,17 +566,17 @@ mod tests {
 
     #[test]
     fn layer_creation() {
-        let mut ufo = crate::Font::load("testdata/mutatorSans/MutatorSansBoldWide.ufo").unwrap();
+        let mut ufo = crate::Font::load("testdata/MutatorSansLightWide.ufo").unwrap();
 
-        let default_layer = ufo.layers.get_or_create("foreground".into());
+        let default_layer = ufo.layers.get_or_create("foreground");
         assert!(!default_layer.is_empty());
         default_layer.clear();
 
-        let background_layer = ufo.layers.get_or_create("background".into());
+        let background_layer = ufo.layers.get_or_create("background");
         assert!(!background_layer.is_empty());
         background_layer.clear();
 
-        let misc_layer = ufo.layers.get_or_create("misc".into());
+        let misc_layer = ufo.layers.get_or_create("misc");
         assert!(misc_layer.is_empty());
         misc_layer.insert_glyph(Glyph::new_named("A"));
 
